@@ -39,7 +39,10 @@ int main (int argc, char **argv) {
   uint32_t *ids = NULL;
   uint32_t *tmp = NULL;
   char *playlistname = NULL;
+  char *rest;
  
+  fprintf(stdout, "libmtp version: " LIBMTP_VERSION_STRING "\n\n");
+
   while ( (opt = getopt(argc, argv, "hn:i:")) != -1 ) {
     switch (opt) {
     case 'h':
@@ -51,7 +54,7 @@ int main (int argc, char **argv) {
         return 1;
       }
       ids = tmp;
-      ids[(idcount-1)] = atoi(strdup(optarg));
+      ids[(idcount-1)] = strtoul(optarg, &rest, 0);
       break;
     case 'n':
       playlistname = strdup(optarg);
@@ -85,7 +88,9 @@ int main (int argc, char **argv) {
   playlist->name = playlistname;
   playlist->no_tracks = idcount;
   playlist->tracks = ids;
-  int ret = LIBMTP_Create_New_Playlist(device,playlist,0);
+  playlist->parent_id = 0;
+  playlist->storage_id = 0;
+  int ret = LIBMTP_Create_New_Playlist(device,playlist);
   if (ret != 0) {
     printf("Couldn't create playlist object\n");
     LIBMTP_Dump_Errorstack(device);
