@@ -20,6 +20,9 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#define _LARGEFILE_SOURCE
+#define _LARGEFILE64_SOURCE
+
 #include <string.h>
 #include <libgen.h>
 #include <sys/stat.h>
@@ -47,18 +50,30 @@ int sendfile_function(char * from_path, char *to_path)
   printf("Sending %s to %s\n",from_path,to_path);
   char *filename;
   uint64_t filesize;
+#ifdef __USE_LARGEFILE64
+  struct stat64 sb;
+#else
   struct stat sb;
+#endif
   LIBMTP_file_t *genfile;
   int ret;
   uint32_t parent_id = 0;
 
+#ifdef __USE_LARGEFILE64
+  if ( stat64(from_path, &sb) == -1 ) {
+#else
   if ( stat(from_path, &sb) == -1 ) {
+#endif
     fprintf(stderr, "%s: ", from_path);
     perror("stat");
     exit(1);
   }
 
+#ifdef __USE_LARGEFILE64
+  filesize = sb.st_size;
+#else
   filesize = (uint64_t) sb.st_size;
+#endif
   filename = basename(from_path);
   parent_id = parse_path (to_path,files,folders);
   if (parent_id == -1) {
@@ -70,14 +85,18 @@ int sendfile_function(char * from_path, char *to_path)
   genfile->filesize = filesize;
   genfile->filename = strdup(filename);
   genfile->filetype = find_filetype (filename);
+  genfile->parent_id = 0;
+  genfile->storage_id = 0;
 
   printf("Sending file...\n");
-  ret = LIBMTP_Send_File_From_File(device, from_path, genfile, progress, NULL, parent_id);
+  ret = LIBMTP_Send_File_From_File(device, from_path, genfile, progress, NULL);
   printf("\n");
   if (ret != 0) {
     printf("Error sending file.\n");
     LIBMTP_Dump_Errorstack(device);
     LIBMTP_Clear_Errorstack(device);
+  } else {
+    printf("New file ID: %d\n", genfile->item_id);
   }
 
   LIBMTP_destroy_file_t(genfile);
